@@ -1,7 +1,7 @@
 """adripy is a set of python tools for manipulating MSC Adams Drill files and data.
 """
-from .utilities import * 
-
+#pylint: disable=wrong-import-position
+import os
 import jinja2
 
 TMPLT_ENV = jinja2.Environment(
@@ -12,9 +12,10 @@ TMPLT_ENV = jinja2.Environment(
     lstrip_blocks=True
 )
 
+from .utilities import * 
 from .tool import DrillTool
 from .event import DrillEvent
 from .string import DrillString
 from .solver_settings import DrillSolverSettings
 from .drillsim import DrillSim
-from .hole import DrillHole
\ No newline at end of file
+from .hole import DrillHole
