@@ -1,19 +1,27 @@
 """Contains the DrillEvent class
 """
 import os
+import logging
 import copy
 import thornpy
 from . import TMPLT_ENV
 from .utilities import read_TO_file, get_cdb_path, get_full_path
 
+# Configure Logging
+LOG = logging.getLogger(__name__)
+
 class DrillEvent():
     """
-    Creates an object with all data necessary to write a drill event.  Once the DrillEvent is instanced ramp parameters must be defined before the DrillEvent is written to an event file.  
+    Creates an object with all data necessary to write a drill event.  Once the DrillEvent is instanced ramp parameters must be defined before the DrillEvent is written to an event file.
 
     Note
     ----
     After instancing the class, at least one simulation step must be added using the `add_simulation_step()` method. Also, at least one ramp must be added for each of the four drilling parameters (i.e. wob, rpm, gpm, rop) using the `add_ramp()` method.  All other parameters in the event file can be specified when the DrillEvent is instanced using kwargs or they can be set later using:
         >>> drill_event.parameters[parameter] = value 
+
+    Note
+    ----
+    WOB should be in lbf    
     
     Attributes
     ----------
@@ -21,8 +29,8 @@ class DrillEvent():
         Dictionary of parameters that make up an Adams Drill string and would be found in an Adams Drill String file (.str).  The keys of the dictionary are the parameter names that would be seen in the string file and the values of the dictionary are the values that would be seen in the string file.
     filename : str
         Name of the event file (.evt) in which this event is stored.  This attribute is initially empty and is populated by the `write_to_file()` method. 
-    """
 
+    """
     _SCALAR_PARAMETERS = [
         'Units',
         'Event_Name',
@@ -79,7 +87,7 @@ class DrillEvent():
     _TABLE_PARAMETERS = [
         'ROTARY_RPM',
         'MOTOR',
-        'PUMP_FLOW',
+        'FLOW_RATE',
         'WOB',
         'ROP',
         'NPERREV',
@@ -89,7 +97,7 @@ class DrillEvent():
     _DEFAULT_PARAMETER_TABLES = {
         'ROTARY_RPM': ((),(),()),
         'MOTOR': ((0,), (1,), (1,)),
-        'PUMP_FLOW': ((),(),()),
+        'FLOW_RATE': ((),(),()),
         'WOB': ((),(),()),
         'ROP': ((),(),()),
         'NPERREV': ((0,),(1,)),
@@ -146,7 +154,7 @@ class DrillEvent():
         Parameters
         ----------
         parameter : str
-            ramp parameter, options are 'TOP_DRIVE', 'WOB', 'PUMP_FLOW', or 'ROP'
+            ramp parameter, options are 'TOP_DRIVE', 'WOB', 'FLOW_RATE', or 'ROP'
         start_time : float
             Start time of the ramp.
         ramp_duration : float
@@ -166,6 +174,10 @@ class DrillEvent():
 
     def write_to_file(self, directory=None, filename=None, cdb=None):
         """Creates an event file from the DrillEvent object.
+
+        Note
+        ----
+        Uses file version 1.0
         
         Parameters
         ----------
@@ -234,7 +246,6 @@ class DrillEvent():
         self.filename = get_cdb_path(filepath)
 
         return self.filename
-
     
     def validate(self):
         """
@@ -250,10 +261,12 @@ class DrillEvent():
         # Check that all parameters exist in the self.parameters dictionary
         for param_name in self._SCALAR_PARAMETERS:
             if param_name not in self.parameters:
+                LOG.critical('%s not found in %s', param_name, self.filename)
                 validated = False        
         
         for param_name in self._TABLE_PARAMETERS:
             if not all([elem for elem in self.parameters[param_name]]):
+                LOG.critical('%s not found in %s', param_name, self.filename)
                 validated = False
             
         return validated          
@@ -299,6 +312,10 @@ class DrillEvent():
     
     def _get_params_from_TO_data(self, tiem_orbit_data):
         """Reads the event parameters out of a dictoinary of Tiem Orbit data generated by :func:`adamspy.adripy.utilities.read_TO_file`
+
+        Note
+        ----
+        Adams 2018 uses *delta* values for the surface parameters where as Adams 2019 uses *at end* values.  :class:`DrillEvent` always uses *delta* values, so any *at end* values are converted in this method.
         
         Parameters
         ----------
@@ -311,6 +328,8 @@ class DrillEvent():
             An event parameter could not be found
 
         """
+        file_version = float(tiem_orbit_data['ADAMS_DRILL_HEADER']['file_version'])
+
         for param in self._SCALAR_PARAMETERS:
             # For each event parameter initialize a found flag
             found = False
@@ -318,13 +337,20 @@ class DrillEvent():
             for block in tiem_orbit_data:
                 # For each block in the TO file
 
-                if param.lower() in tiem_orbit_data[block]:
+                if param.lower() in ['start_time', 'end_time']:
+                    # If finding the End_Time, or Start_Time parameters, only look in the PLOT_4D block
+                    if block.lower() == 'plot_4d':                        
+                        self.parameters[param] = tiem_orbit_data[block][param.lower()]
+                        found = True
+                        break
+
+                elif param.lower() in tiem_orbit_data[block]:
                     # If the parameter is in this block, set the parameter and break the loop
                     self.parameters[param] = tiem_orbit_data[block][param.lower()]
                     found = True
                     break
 
-                else:
+                elif param.lower() != 'end_time':
                     # If the parameter is not in this block, find all the sub blocks 
                     # and look for the parameter inside each sub block
                     sub_blocks = [header for header in tiem_orbit_data[block] if isinstance(tiem_orbit_data[block][header], dict)]
@@ -355,9 +381,8 @@ class DrillEvent():
                 # For each block in the TO file, get the sub blocks
                 sub_blocks = [header for header in tiem_orbit_data[block] if isinstance(tiem_orbit_data[block][header], dict)]
                 
-                # This is a bandaid to make this code backward compatible
-                if 'TOP_DRIVE' in sub_blocks:
-                    
+                # vvvvvvvvvvvv This is a bandaid to make this code backward compatible vvvvvvvvvvvv
+                if 'TOP_DRIVE' in sub_blocks:                    
                     # Replace 'TOP_DRIVE' with 'ROTARY_RPM'
                     for _i in range(len(sub_blocks)):
                         if sub_blocks[_i] == 'TOP_DRIVE':
@@ -366,6 +391,17 @@ class DrillEvent():
 
                     tiem_orbit_data[block]['ROTARY_RPM'] = tiem_orbit_data[block].pop('TOP_DRIVE')
 
+                if 'PUMP_FLOW' in sub_blocks:                    
+                    # Replace 'PUMP_FLOW' with 'FLOW_RATE'
+                    for _i in range(len(sub_blocks)):
+                        if sub_blocks[_i] == 'PUMP_FLOW':
+                            sub_blocks[_i] = 'FLOW_RATE'
+                            break                   
+
+                    tiem_orbit_data[block]['FLOW_RATE'] = tiem_orbit_data[block].pop('PUMP_FLOW')
+                
+                # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+
                 if param.upper()==block and block=='DYNAMICS':
                     # If the parameter is DYNAMICS and so is the current block
                     self.parameters[param][0] = tiem_orbit_data[block]['end_time']
@@ -386,11 +422,47 @@ class DrillEvent():
                 elif param.upper() in sub_blocks:
                     # If the current parameter is in this sub block, add to the parameters dictionary
                     self.parameters[param][0] = tiem_orbit_data[block][param.upper()]['start_time']
-                    self.parameters[param][1] = tiem_orbit_data[block][param.upper()]['ramp_duration']
+
+                    if 'ramp_duration' in tiem_orbit_data[block][param.upper()]:
+                        # If the .evt file uses *ramp duration* notation, no conversion necessary
+                        self.parameters[param][1] = tiem_orbit_data[block][param.upper()]['ramp_duration']
+
+                    elif 'end_time' in tiem_orbit_data[block][param.upper()]:
+                        # If the .evt file uses *at end* notation, convert to *ramp duration* notation
+                        self.parameters[param][1] = [end - start for end, start in zip(tiem_orbit_data[block][param.upper()]['end_time'], tiem_orbit_data[block][param.upper()]['start_time'])]
+
                     if len(self.parameters[param])>2:
                         list_params = [par for par in tiem_orbit_data[block][param.upper()] if isinstance(tiem_orbit_data[block][param.upper()][par], list)]
-                        last_col = [col for col in list_params if col not in ['start_time','ramp_duration']][0]
-                        self.parameters[param][2] = tiem_orbit_data[block][param.upper()][last_col]
+                        last_col = [col for col in list_params if col not in ['start_time','ramp_duration', 'end_time']][0]
+                                                
+                        if 'delta' in last_col:
+                            # If .evt file uses 2018 notation (delta)
+                            self.parameters[param][2] = tiem_orbit_data[block][param.upper()][last_col]
+                        
+                        elif 'at_end' in last_col:
+                            # If .evt file uses 2019 notation (value at end)
+                            at_end_values = tiem_orbit_data[block][param.upper()][last_col]
+
+                            if len(at_end_values) < 2:                                
+                                # If there is only one value (the delta and the end value are the same)
+                                self.parameters[param][2] = at_end_values
+                            
+                            else:
+                                # If there are multiple values, calculate the deltas                                
+                                # The first "delta" value is the same as the first "at end" value
+                                delta_values = at_end_values[:1]
+
+                                # Loop over the "at end" values to calculate the "delta" values
+                                for at_end_value, prev_at_end_value in zip(at_end_values[1:], at_end_values[:-1]):
+                                    delta_values.append(at_end_value - prev_at_end_value)
+
+                                self.parameters[param][2] = delta_values
+                        
+                        # Check if need to convert wob from klbf to lbf
+                        if file_version >= 2.0 and param.lower() == 'wob':
+                            # If this is a file version 2.0 and the parameter is wob, convert from klbf to lbf
+                            self.parameters[param][2] = [p*1000 for p in self.parameters[param][2]]
+
                     found = True
                     break
                 
