@@ -4,6 +4,7 @@ import os
 import copy
 import re
 import shutil
+import thornpy
 from . import TMPLT_ENV
 from .tool import DrillTool
 from .utilities import read_TO_file, get_cdb_location, get_cdb_path, get_full_path, TO_LENGTH_PARAM, isabs
@@ -469,6 +470,36 @@ class DrillString():
         drill_string._get_tools_from_TO_data(filename) #pylint: disable=protected-access
 
         return drill_string
+
+    def check_for_references(self, filename):
+        """Checks for references to `filename` in the string.  Returns the number of references found.
+        
+        Parameters
+        ----------
+        filename : str
+            Filename to find references to.
+        
+        Returns
+        -------
+        int
+            Number of references found.
+
+        """
+        counter = 0
+        for tool in self.tools:
+            if tool['Property_File'].replace('/', '\\') == filename.replace('/', '\\'):
+                counter += 1
+        
+        if 'Property_File' in self.top_drive and self.top_drive['Property_File'].replace('/', '\\') == filename.replace('/', '\\'):
+            counter += 1
+
+        if self.parameters['Event_Property_File'].replace('/', '\\') == filename.replace('/', '\\'):
+            counter += 1
+
+        if self.parameters['Hole_Property_File'].replace('/', '\\') == filename.replace('/', '\\'):
+            counter += 1
+
+        return counter
     
     def validate(self):
         """
@@ -615,7 +646,7 @@ class DrillString():
 
             # If the tool property file is not absolute, make it absolute
             if not isabs(tool_file):
-                tool_file = os.path.join(os.path.split(tiem_orbit_file)[0], os.path.normpath(tool_file))
+                tool_file = os.path.join(os.path.split(tiem_orbit_file)[0], thornpy.utilities.convert_path(tool_file))
             
             # Create a DrillTool object from the tool property file
             tool = DrillTool(tool_file)
@@ -633,7 +664,7 @@ class DrillString():
             equivalent = True if 'equivalent' in tiem_orbit_data[block]['type'].lower() else False
 
             # Add the tool to the tools list
-            self.add_tool(tool, joints=joints, measure=tiem_orbit_data[block]['measure'], group_name=group_name, stack_order=tiem_orbit_data[block]['stack_order'], equivalent=equivalent)
+            self.add_tool(tool, joints=joints, measure=tiem_orbit_data[block]['measure'].lower() in ['yes', 'true'], group_name=group_name, stack_order=tiem_orbit_data[block]['stack_order'], equivalent=equivalent)
         
         # -----------------
         # Add the top drive
@@ -643,7 +674,7 @@ class DrillString():
 
         # If the top drive property file is not absolute, make it absolute
         if not isabs(top_drive_file):
-            top_drive_file = os.path.join(os.path.split(tiem_orbit_file)[0], os.path.normpath(top_drive_file))
+            top_drive_file = os.path.join(os.path.split(tiem_orbit_file)[0], thornpy.utilities.convert_path(top_drive_file))
         
         # Create a DrillTool object from the property file
         top_drive = DrillTool(top_drive_file)
@@ -702,15 +733,20 @@ class DrillString():
                     param_value = []
                 else:
                     raise ValueError(f'{param} not found!')
-            
-            # If the parameter is a relative filename, make it absolute
+                        
             if param in self._FILENAME_PARAMETERS and not isabs(param_value):
-                param_value = os.path.join(os.path.split(get_cdb_path(tiem_orbit_file))[0], os.path.normpath(param_value))
+                # If the parameter is a relative filename, make it absolute
+                param_value = os.path.join(os.path.split(get_cdb_path(tiem_orbit_file))[0], thornpy.utilities.convert_path(param_value))
+            
+            elif param in self._FILENAME_PARAMETERS:
+                # If the parameters is an absolute filename, make sure it is formatted correctly.
+                param_value = thornpy.utilities.convert_path(param_value)
+
+            
+
             
             # store the parameter in the self.parameters dict
             self.parameters[param] = param_value
 
-                
-
 class DrillStringError(Exception):
     pass
